## Detecting local adaptation in North American wolves using Redundancy Analysis (RDA) ##

# Annotated code from Forester et al. (in revision) Molecular Ecology
# Original data are from Schweizer et al. (2015) Molecular Ecology; Dryad doi:10.5061/dryad.c9b25


setwd("/path/to/wolf/data")

library(adegenet) # Used to load genetic data from PLINK format
library(psych)    # Used to investigate correlations among predictors
library(vegan)    # Used to run RDA


# Read in and prepare the genetic data for analysis
# -------------------------------------------------

# Data were downloaded from the Schweizer et al. Dryad repository and converted from .tped to .raw format 

dat <- read.PLINK("wolf.raw", parallel=F)
dat                    # 94 individuals; 42,587 SNPs; 3.06% NA
gen <- as.matrix(dat)  # We just want a matrix of genotypes


# RDA requires complete data frames with no missing data

sum(is.na(gen)) # 122630 NAs

# Here, we will use a simple approach to imputing missing values:
# we will impute NAs using the most common genotype at each SNP across all individuals

gen.imp <- apply(gen, 2, function(x) replace(x, is.na(x), as.numeric(names(which.max(table(x))))))
sum(is.na(gen.imp)) # No NAs

# We could also use this imputation approach within ecotypes (rather than across all individuals)

# Other promising imputation methods for species lacking a reference genome include:
# Using ancestry values from "snmf" in the LEA package (Frichot & Francois 2015, Methods in Ecol & Evol)
# LinkImpute (Money et al. 2015, G3)



# Read in and screen the environmental predictors
# -----------------------------------------------

env <- read.csv("EnvironmentalData_6pops_94indiv.csv")

env$land_cover <- as.factor(env$land_cover)
env$individual <- as.character(env$individual)
str(env)

identical(rownames(gen.imp), env[,1])  # confirming genotypes and env data are in the same order

# RDA is a regression-based method, and so is subject to problems when using highly correlated predictors.
# Please see Dormann et al. (2013) Ecography for a discussion of these issues.
# Generally, the |r| > 0.7 "rule of thumb" is a good guideline for removing correlated predictors.
# We will also check for multicollinearity below using Variance Inflation Factors.

# Variable reduction should be guided by an ecological interpretation of the relevance of possible predictors.
# Below, we show one option for variable reduction:

pairs.panels(env[,5:16], scale=T)  # see ?pairs.panels for a description of these biplots
# correlation coefficients are in the upper right diagonal, with their size scaled to the value of |r|

table(env$land_cover) # let's look at land cover, which is a factor
# 0  1  5  7  8 10 12 14 15 16 
# 3 23  7 40  7  4  2  1  1  6 

# The distribution of factor levels is highly skewed towards classes 1 & 7, & we have many classes
# with small numbers of individuals; these characteristics make it unlikely to be highly informative.
env <- env[,-13] # remove land cover

pairs.panels(env[,5:15], scale=T)

env <- env[,-12] # remove precip coldest quarter: 0.97 corr with annual precip
pairs.panels(env[,5:14], scale=T)

env <- env[,-c(8,9)] # remove maxT_warm, minT_cold; covers remaining strong correlations in a small # of predictors
pairs.panels(env[,5:12], scale=T)


# Try this reduced set of predictors:
pred <- env[,5:12]
colnames(pred) <- c("AMT","MDR","sdT","AP","cvP","NDVI","Elev","Tree")



# Run the RDA
# -----------

# We highly recommend the following book for more information on interpreting RDA output from vegan:
# Borcard et al. (2011) Numerical Ecology with R

# Note: write out the formula when using factors as predictors
# or when using significance tests by axis (below)

wolf.rda <- rda(gen.imp ~ ., data=pred, scale=T)
wolf.rda

vif.cca(wolf.rda) # All are below 10 and most are below 5 = multicollinearity shouldn't be a problem

signif.full <- anova.cca(wolf.rda)             # default is permutation=999
signif.axis <- anova.cca(wolf.rda, by="axis")  # takes a while!

signif.full  # full model is significant...but that doesn't tell us much
signif.axis  # first four (of 8) constrained axes are significant (p < 0.01)



# Plot the RDA
# ------------

# Simple plots
plot(wolf.rda, scaling=3)  # 3 = symmetrical scaling (SNP & indiv scores are scaled by the square root of the eigenvalues)
# See Borcard et al. 2011 or vegan help for more information on scaling RDA plots

plot(wolf.rda, c(1,3), scaling=3)  # axes 1 and 3


# Nicer plots:
dat@pop
levels(dat@pop) <- c("Western Forest","Boreal Forest","Arctic","High Arctic","British Columbia","Atlantic Forest")
colorby <- dat@pop
bg <- c("#ff7f00","#1f78b4","#ffff33","#a6cee3","#33a02c","#e31a1c")

# axes 1 & 2
plot(wolf.rda, type="n", scaling=3)
points(wolf.rda, display="species", pch=20, cex=0.7, col="gray32", scaling=3)               # the snps
points(wolf.rda, display="sites", pch=21, cex=1.3, col="gray32",scaling=3, bg=bg[colorby])  # the wolves
text(wolf.rda, scaling = 3, display = "bp", col="#0868ac", cex=1.1)                         # the predictors
legend("bottomright", legend = levels(colorby), bty = "n", col="gray32", pch = 21, cex=1.3, pt.bg = bg)

# axes 1 & 3
plot(wolf.rda, type="n", scaling=3, choices=c(1,3))
points(wolf.rda, display="species", pch=20, cex=0.7, col="gray32", scaling=3, choices=c(1,3))
points(wolf.rda, display="sites", pch=21, cex=1.3, col="gray32",scaling=3, bg=bg[colorby], choices=c(1,3))
text(wolf.rda, scaling = 3, display = "bp", col="#0868ac", cex=1.1, choices=c(1,3))
legend("topleft", legend = levels(colorby), bty = "n", col="gray32", pch = 21, cex=1.3, pt.bg = bg)



# Pull the loadings
# -----------------

load.rda <- summary(wolf.rda)$species[,1:4]   # RDA loadings

par(mfrow=c(2,2))
hist(load.rda[,1])
hist(load.rda[,2])
hist(load.rda[,3]) 
hist(load.rda[,4])  # loadings relatively normally distributed on each significant axis
par(mfrow=c(1,1))

outliers <- function(x,z){
  lims <- mean(x) + c(-1, 1) * z * sd(x)     # find loadings +/-z sd from mean loading     
  x[x < lims[1] | x > lims[2]]               # locus names in these tails
}

# Start with 3 standard deviation cutoff. As with all cutoffs, this can be modified to
# reflect the goals of the analysis and our tolerance for true positives vs. false positives

cand1 <- outliers(load.rda[,1],3) # 171
cand2 <- outliers(load.rda[,2],3) # 270
cand3 <- outliers(load.rda[,3],3) # 163
cand4 <- outliers(load.rda[,3],3) # 163

ncand <- length(cand1)+length(cand2)+length(cand3)+length(cand4)  # 767 candidates

# Make a data frame with the axis, SNP name, loading, & correlation with each predictor
cand1 <- cbind.data.frame(rep(1,times=length(cand1)), names(cand1), unname(cand1)); colnames(cand1) <- c("axis","snp","loading")
cand2 <- cbind.data.frame(rep(2,times=length(cand2)), names(cand2), unname(cand2)); colnames(cand2) <- c("axis","snp","loading")
cand3 <- cbind.data.frame(rep(3,times=length(cand3)), names(cand3), unname(cand3)); colnames(cand3) <- c("axis","snp","loading")
cand4 <- cbind.data.frame(rep(4,times=length(cand4)), names(cand4), unname(cand4)); colnames(cand4) <- c("axis","snp","loading")

cand <- rbind(cand1, cand2, cand3, cand4)
cand$snp <- as.character(cand$snp)

foo <- matrix(nrow=(ncand), ncol=8)  # 8 predictors
colnames(foo) <- c("ann_mean_temp","mean_diurnal_range","temp_seasonality","ann_precip","precip_seasonality","ndvi","elev","percent_tree_cover")

for (i in 1:length(cand$snp)) {
  nam <- cand[i,2]
  snp.gen <- gen.imp[,nam]
  foo[i,] <- apply(pred,2,function(x) cor(x,snp.gen))
}

cand <- cbind.data.frame(cand,foo)  # 767 detections with their correlation wtih our 8 predictors
head(cand)


# Investigate the candidates 
# --------------------------

# SNPs detected on multiple axes:
cand$snp[duplicated(cand$snp)] # 211 duplicates

foo <- cbind(cand$axis, duplicated(cand$snp)) 
table(foo[foo[,1]==1,2]) # no duplicates on axis 1
table(foo[foo[,1]==2,2]) # 48 duplicates on axis 2
table(foo[foo[,1]==3,2]) # no duplicates on axis 3
table(foo[foo[,1]==4,2]) # 163 (all detections) duplicates on axis 4

cand <- cand[!duplicated(cand$snp),] # remove duplicate detections


# Let's see which of the predictors each candidate SNP is most strongly correlated with:
for (i in 1:length(cand$snp)) {
  bar <- cand[i,]
  cand[i,12] <- names(which.max(abs(bar[4:11]))) # gives the variable
  cand[i,13] <- max(abs(bar[4:11]))              # gives the correlation
}

colnames(cand)[12] <- "predictor"
colnames(cand)[13] <- "correlation"

write.csv(cand,"wolf_cand_4axes.csv", row.names=F)

table(cand$predictor) 
table(cand$axis)      



# Plot the SNPs
# -------------

sel <- cand$snp
env <- cand$predictor
env[env=="ann_precip"] <- '#1f78b4'
env[env=="precip_seasonality"] <- '#a6cee3'
env[env=="mean_diurnal_range"] <- '#6a3d9a'
env[env=="ann_mean_temp"] <- '#e31a1c'
env[env=="ndvi"] <- '#33a02c'
env[env=="elev"] <- '#ffff33'
env[env=="temp_seasonality"] <- '#fb9a99'
env[env=="percent_tree_cover"] <- '#b2df8a'

# color by predictor:
all.snp <- rownames(wolf.rda$CCA$v)
test <- all.snp

for (i in 1:length(sel)) {
  foo <- match(sel[i],test)
  test[foo] <- env[i]
}

test[grep("chr",test)] <- '#f1eef6'
empty <- test
empty[grep("#f1eef6",empty)] <- rgb(0,1,0, alpha=0) # transparent
empty.outline <- ifelse(empty=="#00FF0000","#00FF0000","gray32")

# axes 1 & 2
plot(wolf.rda, type="n", scaling=3, xlim=c(-1,1), ylim=c(-1,1))
points(wolf.rda, display="species", pch=21, cex=1.3, col="gray32", bg=test, scaling=3)
points(wolf.rda, display="species", pch=21, cex=1.3, col=empty.outline, bg=empty, scaling=3)
text(wolf.rda, scaling = 3, display = "bp", col="#0868ac", cex=1.1)
bg <- c('#1f78b4','#a6cee3','#6a3d9a','#e31a1c','#33a02c','#ffff33','#fb9a99','#b2df8a')
legend("bottomright", legend = c("AP","cvP","MDR","AMT","NDVI","Elev","sdT","Tree"), bty = "n", col="gray32", pch = 21, cex=1.3, pt.bg = bg)

# axes 1 & 3
plot(wolf.rda, type="n", scaling=3, xlim=c(-1,1), ylim=c(-1,1), choices=c(1,3))
points(wolf.rda, display="species", pch=21, cex=1.3, col="gray32", bg=test, scaling=3, choices=c(1,3))
points(wolf.rda, display="species", pch=21, cex=1.3, col=empty.outline, bg=empty, scaling=3, choices=c(1,3))
text(wolf.rda, scaling = 3, display = "bp", col="#0868ac", cex=1.1, choices=c(1,3))
bg <- c('#1f78b4','#a6cee3','#6a3d9a','#e31a1c','#33a02c','#ffff33','#fb9a99','#b2df8a')
legend("bottomright", legend = c("AP","cvP","MDR","AMT","NDVI","Elev","sdT","Tree"), bty = "n", col="gray32", pch = 21, cex=1.3, pt.bg = bg)
